/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj;

import javassist.gluonj.pc.PointcutNode;
/*
 * How to define a new pointcut designator (pcd).
 *
 * Edit Pcd and Pointcut.
 * Define a node class in javassist.gluonj.pc.
 * It must extend PointcutNode.  PointcutVisitor must be also edited.
 * Then edit Matcher and its subclass in the Gluon class. 
 * Finally, WeaveEditor in AdvcieWeaver must be edited.
 */

public class Pcd {
    static class NotPointcut extends Pointcut {
        NotPointcut() { super(null); }
        protected boolean isNot() { return true; }
        protected void setTree(PointcutNode node) {
            Pcd.not = new NotPointcut();
            super.setTree(node);
        }
    };

    public static Pointcut not = new NotPointcut(); 

    public static Pointcut define(String aliasName, String expr) {
        return new AliasedPcExpr(aliasName, expr);
    }

    public static Pointcut expr(Pointcut pc) {
        return new Pointcut(null).expr(pc);
    }

    public static Pointcut within(String className) {
        return new Pointcut(null).within(className);
    }

    public static Pointcut call(String methodName) {
        return new Pointcut(null).call(methodName);
    }

    public static Pointcut set(String fieldName) {
        return new Pointcut(null).set(fieldName);
    }

    public static Pointcut get(String fieldName) {
        return new Pointcut(null).get(fieldName);
    }

    /**
     * AspectJ's if pointcut designator.
     */
    public static Pointcut when(String expr) {
        return new Pointcut(null).when(expr);
    }

    public static Pointcut cflow(String methodName) {
        return new Pointcut(null).cflow(methodName);
    }

    /**
     * A factory method for plugins.
     */
    protected static Pointcut make() {
        return new Pointcut(null);
    }
}
