/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj;

import javassist.CtField;
import javassist.NotFoundException;

public class WeaveException extends Exception {
    private String where = null;

    public String getMessage() {
        String m = super.getMessage();
        if (where != null)
            m = where + ", " + m;

        return m;
    }

    public void setWhere(String w) { where = w; }

    public WeaveException(Exception e) {
        super(e.getMessage(), e);
    }

    public WeaveException(ClassNotFoundException e) {
        super("class not found: " + e.getMessage(), e);
    }

    public WeaveException(NotFoundException e) {
        super("not found: " + e.getMessage(), e);
    }

    public WeaveException(String m, Exception e) {
        super(m, e);
    }

    public WeaveException(String m) {
        super(m);
    }

    public WeaveException(String m, CtField f) {
        super(m + ": " + f.getName() + " in "
                + f.getDeclaringClass().getName());
    }
}
