/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.pc;

import java.lang.reflect.Method;

import javassist.CtClass;
import javassist.gluonj.WeaveException;
import javassist.gluonj.util.SimplePcNode;

public class PrepareVisitor implements PointcutVisitor {
    protected PatternParser parser;

    public PrepareVisitor(PatternParser p) {
        parser = p;
    }

    public PatternParser getParser() { return parser; }

    public void visit(AndPc n) throws WeaveException {
        n.first().accept(this);
        AndPc r = n.rest();
        if (r != null)
            r.accept(this);
    }

    public void visit(OrPc n) throws WeaveException {
        n.first().accept(this);
        OrPc r = n.rest();
        if (r != null)
            r.accept(this);
    }

    public void visit(NotPc n) throws WeaveException {
        n.getExpr().accept(this);
    }

    public void visit(WithinPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(CallPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(GetPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(SetPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(IfPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(CflowPc pc) throws WeaveException {
        pc.prepare(parser);
    }

    public void visit(SimplePcNode pc) throws WeaveException {
        if (overriddenInspect(pc))
            parser.getGlue().registerInspector(pc);

        pc.prepare(parser);
    }

    public static boolean overriddenInspect(SimplePcNode pc) {
        try {
            Method m = pc.getClass().getMethod("inspect", new Class[] { CtClass.class });
            if (!m.getDeclaringClass().equals(SimplePcNode.class))
                return true;
        }
        catch (NoSuchMethodException e) {}
        return false;
    }
}
