/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

import java.util.List;

import javassist.CtBehavior;
import javassist.gluonj.WeaveException;
import javassist.gluonj.pc.AndPc;
import javassist.gluonj.pc.CallPc;
import javassist.gluonj.pc.CflowPc;
import javassist.gluonj.pc.GetPc;
import javassist.gluonj.pc.IfPc;
import javassist.gluonj.pc.NotPc;
import javassist.gluonj.pc.OrPc;
import javassist.gluonj.pc.PointcutVisitor;
import javassist.gluonj.pc.SetPc;
import javassist.gluonj.pc.WithinPc;
import javassist.gluonj.util.SimplePcNode;

/**
 * CflowCollector collects all CflowPc nodes in a pointcut
 * expression.  Those nodes are used for inserting code
 * fragments for monitoring control flow.
 *
 * @see CflowPc#instrument(CtBehavior)
 */
public class CflowCollector implements PointcutVisitor {
    protected List cflows;

    public CflowCollector(List list) {
        cflows = list;
    }

    public List getCflows() { return cflows; }

    public void visit(AndPc node) throws WeaveException {
        node.first().accept(this);
        AndPc rest = node.rest();
        if (rest != null)
            rest.accept(this);
    }

    public void visit(OrPc node) throws WeaveException {
        node.first().accept(this);
        OrPc rest = node.rest();
        if (rest != null)
            rest.accept(this);
    }

    public void visit(NotPc n) throws WeaveException {
        n.getExpr().accept(this);
    }

    public void visit(CflowPc node) throws WeaveException {
        cflows.add(node);
    }
    
    public void visit(WithinPc n) throws WeaveException {}
    public void visit(CallPc n) throws WeaveException {}
    public void visit(GetPc n) throws WeaveException {}
    public void visit(SetPc n) throws WeaveException {}
    public void visit(IfPc n) throws WeaveException {}
    public void visit(SimplePcNode pc) throws WeaveException {}
}
