/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

import javassist.CtBehavior;
import javassist.CtConstructor;
import javassist.gluonj.WeaveException;
import javassist.gluonj.pc.PcPattern;
import javassist.gluonj.pc.WithinPc;

/**
 * A super class of CallMatcher, FieldMatcher, ...
 */
public abstract class ExprMatcher extends Matcher {
    private String withinClass;
    private String withinMethodName, withinMethodSignature;

    protected ExprMatcher(CtBehavior where) {
        super();
        withinClass = where.getDeclaringClass().getName();
        withinMethodName = where.getName();
        withinMethodSignature = where.getSignature();
        if (where instanceof CtConstructor)
            if (((CtConstructor)where).isConstructor())
                withinMethodName = AdviceWeaver.CONSTRUCTOR;
    }

    public void visit(WithinPc pc) throws WeaveException {
        PcPattern pp = pc.getPattern();
        if (pp.isMethod())
            result = pp.matchMethod(withinClass, withinMethodName, withinMethodSignature);
        else
            result = pp.matchClass(withinClass);
    }
}
