/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

import javassist.expr.FieldAccess;
import javassist.expr.Expr;
import javassist.gluonj.WeaveException;
import javassist.gluonj.pc.GetPc;
import javassist.gluonj.pc.SetPc;
import javassist.gluonj.util.SimplePcNode;

/**
 * A Matcher customized for field-access join points.
 */
public class FieldMatcher extends ExprMatcher {
    protected FieldAccess joinPoint;
    protected String declClass;
    protected String fieldName;
    protected boolean isRead;

    public FieldMatcher(FieldAccess fa) {
        super(fa.where());
        joinPoint = fa;
        declClass = fa.getClassName();
        fieldName = fa.getFieldName();
        isRead = fa.isReader();
    }

    public Expr getJoinPoint() {
        return joinPoint;
    }

    public void visit(SetPc pc) throws WeaveException {
        if (isRead)
            result = false;
        else
            result = pc.getPattern().matchField(declClass, fieldName);
    }

    public void visit(GetPc pc) throws WeaveException {
        if (isRead)
            result = pc.getPattern().matchField(declClass, fieldName);
        else
            result = false;
    }

    public void visit(SimplePcNode pc) throws WeaveException {
        Residue[] r = new Residue[1];
        result = pc.match(joinPoint, r);
        residue = r[0];
    }
}

