/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

import javassist.expr.NewExpr;
import javassist.expr.Expr;
import javassist.gluonj.WeaveException;
import javassist.gluonj.pc.CallPc;
import javassist.gluonj.util.SimplePcNode;

/**
 * A Matcher customized for object-creation join points.
 */
public class NewMatcher extends ExprMatcher {
    private NewExpr joinPoint;
    private String declClass;
    private String name, descriptor;

    public NewMatcher(NewExpr expr) {
        super(expr.where());
        joinPoint = expr;
        declClass = expr.getClassName();
        name = AdviceWeaver.CONSTRUCTOR;
        descriptor = expr.getSignature();
    }

    public Expr getJoinPoint() {
        return joinPoint;
    }

    public void visit(CallPc pc) throws WeaveException {
        result = pc.getPattern().matchMethod(declClass, name, descriptor);
    }

    public void visit(SimplePcNode pc) throws WeaveException {
        Residue[] r = new Residue[1];
        result = pc.match(joinPoint, r);
        residue = r[0];
    }
}
