/*
 * GluonJ, An Aspect-Oriented Programming System for Java.
 * Copyright (C) 2006-2007 Shigeru Chiba. All Rights Reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License.  Alternatively, the contents of this file may be used under
 * the terms of the GNU Lesser General Public License Version 2.1 or later.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 */
package javassist.gluonj.weave;

import javassist.CtClass;
import javassist.ClassPool;
import javassist.NotFoundException;
import javassist.gluonj.WeaveException;
import java.io.ByteArrayInputStream;
import java.io.IOException;

/**
 * A refine class.
 */
public class Refiner {
    public CtClass clazz;           // @Refine class
    public String glueName;
    public boolean privileged;
    public String targetClass;      // may be another @Refine

    // set by Weaver.loadRefines()
    public CtClass realTarget;
    public Refiner parent;          // if the super class of this @Refine
                                    // is also @Refine

    private byte[] backup;

    public Refiner(CtClass refine, String glue, boolean privilege,
                   String target) {
        clazz = refine;
        glueName = glue;
        this.privileged = privilege;
        targetClass = target;
        backup = null;
        realTarget = null;
        parent = null;
    }

    public void saveBackup(byte[] b) {
        backup = b;
    }

    public void revert() throws WeaveException {
        ClassPool cp = clazz.getClassPool();
        String name = clazz.getName();
        clazz.detach();
        try {
            if (backup == null)
                clazz = cp.get(name);
            else
                clazz = cp.makeClass(new ByteArrayInputStream(backup));
        }
        catch (NotFoundException e) {
            throw new WeaveException("lost a class file: " + name);
        }
        catch (IOException e) {
            throw new WeaveException("broken class file: " + name);
        }
    }
}
